@@ -25,7 +25,7 @@
 ######################################################################
 
 # General modules
-import os, re, logging
+import os, re, logging, shutil
 
 logger = logging.getLogger()
 
@@ -101,25 +101,19 @@ def write_authz_file_groups(sbox):
 
 def verify_get(test_area_url, path, user, pw,
                expected_status, expected_body, headers):
-  import httplib
-  from urlparse import urlparse
   import base64
 
   req_url = test_area_url + path
 
-  loc = urlparse(req_url)
-
-  if loc.scheme == 'http':
-    h = httplib.HTTPConnection(loc.hostname, loc.port)
-  else:
-    h = httplib.HTTPSConnection(loc.hostname, loc.port)
+  h = svntest.main.create_http_connection(req_url, 0)
 
   if headers is None:
     headers = {}
 
   if user and pw:
       auth_info = user + ':' + pw
-      headers['Authorization'] = 'Basic ' + base64.b64encode(auth_info)
+      user_pw = base64.b64encode(auth_info.encode()).decode()
+      headers['Authorization'] = 'Basic ' + user_pw
   else:
       auth_info = "anonymous"
 
@@ -138,6 +132,8 @@ def verify_get(test_area_url, path, user, pw,
 
   if expected_body:
       actual_body = r.read()
+      if isinstance(expected_body, str) and not isinstance(actual_body, str):
+        actual_body = actual_body.decode()
       if expected_body != actual_body:
         logger.warn("Expected body:")
         logger.warn(expected_body)
@@ -1047,6 +1043,73 @@ def authn_sallrall(sbox):
   verify_gets(test_area_url, sallrall_tests)
 
 
+@SkipUnless(svntest.main.is_ra_type_dav)
+def repos_relative_access_file(sbox):
+  "repos-relative access file"
+
+  sbox.build()
+
+  test_area_url = sbox.repo_url.replace('/svn-test-work/repositories/',
+                                        '/authz-test-work/in-repos-authz/')
+
+  svntest.main.write_authz_file(sbox, {"/": "", "/A": "%s = rw" % user1})
+  shutil.move(sbox.authz_file, os.path.join(sbox.wc_dir, 'authz'))
+  sbox.simple_add('authz')
+  svntest.actions.run_and_verify_svn(None, [], 'relocate',
+      sbox.file_protocol_repo_url(), sbox.wc_dir)
+  sbox.simple_commit(message="adding in-repository authz rules file")
+
+  in_repos_authz_tests = (
+                 { 'path': '', 'status': 401, },
+                 { 'path': '/authz', 'status': 401, },
+                 { 'path': '/authz', 'user' : user1, 'pw' : user1_pass,
+                   'status': 403, },
+                 { 'path': '/A', 'user' : user1, 'pw' : user1_pass,
+                   'status': 301, },
+                 { 'path': '/A/', 'user' : user1, 'pw' : user1_pass,
+                   'status': 200, },
+  )
+
+  verify_gets(test_area_url, in_repos_authz_tests)
+
+# test for the bug also known as CVE-2020-17525
+@SkipUnless(svntest.main.is_ra_type_dav)
+def nonexistent_repos_relative_access_file(sbox):
+  "repos-relative access file with bad repository URL"
+
+  sbox.build()
+
+  test_area_url = sbox.repo_url.replace('/svn-test-work/repositories/',
+                                        '/authz-test-work/in-repos-authz/')
+
+  # Construct a bad test-area URL to see what happens if we attempt to access
+  # a repository in a subdirectory which does not exist in SVNParentPath.
+  # This used to crash the server with a NULL-pointer dereference upon
+  # unauthenticated access.
+  test_area_url += '-this/does/not/exist'
+
+  svntest.main.write_authz_file(sbox, {"/": "", "/A": "%s = rw" % user1})
+  shutil.move(sbox.authz_file, os.path.join(sbox.wc_dir, 'authz'))
+  sbox.simple_add('authz')
+  svntest.actions.run_and_verify_svn(None, [], 'relocate',
+      sbox.file_protocol_repo_url(), sbox.wc_dir)
+  sbox.simple_commit(message="adding in-repository authz rules file")
+
+  # access is denied across the board since this repository does not exist
+  in_repos_authz_tests = (
+                 { 'path': '', 'status': 401, },
+                 { 'path': '/authz', 'status': 401, },
+                 { 'path': '/authz', 'user' : user1, 'pw' : user1_pass,
+                   'status': 403, },
+                 { 'path': '/A', 'user' : user1, 'pw' : user1_pass,
+                   'status': 403, },
+                 { 'path': '/A/', 'user' : user1, 'pw' : user1_pass,
+                   'status': 403, },
+  )
+
+  verify_gets(test_area_url, in_repos_authz_tests)
+
+
 ########################################################################
 # Run the tests
 
@@ -1062,6 +1125,8 @@ test_list = [ None,
               authn_group,
               authn_sallrany,
               authn_sallrall,
+              repos_relative_access_file,
+              nonexistent_repos_relative_access_file,
              ]
 serial_only = True
 
